@@ -19,12 +19,12 @@
 # }
 #
 # \usage{
-#   rowRanges(x, na.rm=FALSE, ...)
-#   colRanges(x, na.rm=FALSE, ...)
-#   rowMins(x, na.rm=FALSE, ...)
-#   colMins(x, na.rm=FALSE, ...)
-#   rowMaxs(x, na.rm=FALSE, ...)
-#   colMaxs(x, na.rm=FALSE, ...)
+#   @usage rowRanges
+#   @usage colRanges
+#   @usage rowMins
+#   @usage colMins
+#   @usage rowMaxs
+#   @usage colMaxs
 # }
 #
 # \arguments{
@@ -51,7 +51,7 @@
 # @author "HB"
 #
 # \seealso{
-#   @see "rowOrderStats" and @see "rowRanges".
+#   @see "rowOrderStats" and @see "base::pmin.int".
 # }
 #
 # @keyword array
@@ -63,47 +63,93 @@ setGeneric("rowRanges", function(x, na.rm=FALSE, ...) {
   standardGeneric("rowRanges")
 })
 
+setGeneric("colRanges", function(x, na.rm=FALSE, ...) {
+  standardGeneric("colRanges")
+})
+
 setMethod("rowRanges", signature(x="matrix"), function(x, na.rm=FALSE, ...) {
-  # Use the much faster rowOrderStats() if possible
-  if (!anyMissing(as.vector(x))) {
-    xRange <- rowOrderStats(x, which=1);
-    xRange <- c(xRange, rowOrderStats(x, which=ncol(x)));
-    dim(xRange) <- c(nrow(x), 2L);
+  .rowRanges(x, na.rm=na.rm, which=1:2, ...);
+}) # rowRanges()
+
+setMethod("colRanges", signature(x="matrix"), function(x, na.rm=FALSE, ...) {
+  .colRanges(x, na.rm=na.rm, which=1:2, drop=FALSE, ...);
+}) # colRanges()
+
+
+rowMins <- function(x, na.rm=FALSE, ...) {
+  .rowRanges(x, na.rm=na.rm, which=1L, drop=TRUE, ...);
+}
+
+rowMaxs <- function(x, na.rm=FALSE, ...) {
+  .rowRanges(x, na.rm=na.rm, which=2L, drop=TRUE, ...);
+}
+
+colMins <- function(x, na.rm=FALSE, ...) {
+  .colRanges(x, na.rm=na.rm, which=1L, drop=TRUE, ...);
+}
+
+colMaxs <- function(x, na.rm=FALSE, ...) {
+  .colRanges(x, na.rm=na.rm, which=2L, drop=TRUE, ...);
+}
+
+.rowRanges <- function(x, na.rm=FALSE, which=1:2, drop=TRUE, ...) {
+  na.rm <- as.logical(na.rm);
+  dim <- dim(x);
+  nrow <- dim[1L];
+  ncol <- dim[2L];
+
+  # Default range is (+Inf,-Inf). See explanation in help(min)
+
+  # Special cases: Nx0 and Nx1 matrices
+  if (ncol == 0L) {
+    xRange <- matrix(c(Inf,-Inf), nrow=nrow, ncol=2L, byrow=TRUE);
+    xRange <- xRange[,which,drop=drop];
+    return(xRange);
+  } else if (ncol == 1L) {
+    xValues <- x[,1L,drop=TRUE];
+    xRange <- matrix(xValues, nrow=nrow, ncol=2L);
+    if (na.rm) {
+      todo <- which(is.na(xValues));
+      ntodo <- length(todo);
+      if (ntodo > 0L) {
+        xRange[todo,] <- matrix(c(Inf,-Inf), nrow=ntodo, ncol=2L, byrow=TRUE);
+      }
+    }
+    xRange <- xRange[,which,drop=drop];
     return(xRange);
   }
 
-  na.rm <- as.logical(na.rm);
-  ncol <- ncol(x);
-  nrow <- nrow(x);
+  min <- (is.element(1L, which));
+  max <- (is.element(2L, which));
 
-  if (ncol == 0) {
-    # Default range is (+Inf,-Inf).  See explanation in help(min)
-    xRange <- matrix(c(Inf,-Inf), nrow=ncol, ncol=2, byrow=TRUE);
-    return(xRange);
-  } else if (ncol == 1) {
-    xRange <- matrix(x[,1,drop=TRUE], nrow=nrow, ncol=2);
-    if (na.rm) {
-      todo <- which(is.na(xRange[,1]));
-      if (length(todo) > 0) {
-        xRange[todo,] <- matrix(c(Inf,-Inf),
-                             nrow=length(todo), ncol=2, byrow=TRUE);
-      }
+  # Use the much faster rowOrderStats() if possible
+  if (!anyMissing(x)) {
+    xRange <- NULL;
+    if (min) {
+      xRange <- c(xRange, rowOrderStats(x, which=1L))
+    }
+    if (max) {
+      xRange <- c(xRange, rowOrderStats(x, which=ncol));
+    }
+    if (!drop || length(which) > 1L) {
+      dim(xRange) <- c(nrow(x), length(which));
     }
     return(xRange);
   }
 
+
   # Default range is (+Inf,-Inf).  See explanation in help(min)
-  xRange <- matrix(c(Inf,-Inf), nrow=nrow, ncol=2, byrow=TRUE);
+  xRange <- matrix(c(Inf,-Inf), nrow=nrow, ncol=2L, byrow=TRUE);
 
   if (na.rm) {
     anyMissing <- TRUE;  # Either none or both of xMin & xMax are NAs.
     for (cc in 1:ncol) {
-      xValues <- x[,cc];
+      xValues <- x[,cc,drop=TRUE];
 
       # Always update the elements with missing ranges.
       if (anyMissing) {
-        todo <- is.na(xRange[,1]);  # Don't need column 2.
-        anyMissing <- (sum(todo) > 0);
+        todo <- is.na(xRange[,1L]);  # Don't need column 2.
+        anyMissing <- (sum(todo) > 0L);
         if (anyMissing) {
           xRange[todo,1:2] <- xValues[todo];
         }
@@ -118,21 +164,25 @@ setMethod("rowRanges", signature(x="matrix"), function(x, na.rm=FALSE, ...) {
       idxs <- which(todo);
       xValues <- xValues[idxs];
 
-      # Identify smaller values
-      isExtreme <- (xValues < xRange[idxs,1]);
-      xRange[idxs[isExtreme],1] <- xValues[isExtreme];
+      # Identify smaller values?
+      if (min) {
+        isExtreme <- (xValues < xRange[idxs,1L]);
+        xRange[idxs[isExtreme],1L] <- xValues[isExtreme];
+      }
 
-      # Identify greater values
-      isExtreme <- (xValues > xRange[idxs,2]);
-      xRange[idxs[isExtreme],2] <- xValues[isExtreme];
+      # Identify greater values?
+      if (max) {
+        isExtreme <- (xValues > xRange[idxs,2L]);
+        xRange[idxs[isExtreme],2L] <- xValues[isExtreme];
+      }
     }
   } else {
     for (cc in 1:ncol) {
       # Update only elements with non-missing extremes
-      idxs <- which(!is.na(xRange[,1]));
+      idxs <- which(!is.na(xRange[,1L]));
 
       # Done?
-      if (length(idxs) == 0)
+      if (length(idxs) == 0L)
         break;
 
       # Assign NAs?
@@ -144,21 +194,27 @@ setMethod("rowRanges", signature(x="matrix"), function(x, na.rm=FALSE, ...) {
       idxs <- idxs[!isMissing];
 
       # Done?
-      if (length(idxs) == 0)
+      if (length(idxs) == 0L)
         break;
 
       xValues <- x[idxs,cc];
 
-      # Identify smaller values
-      isExtreme <- (xValues < xRange[idxs,1]);
-      xRange[idxs[isExtreme],1] <- xValues[isExtreme];
+      # Identify smaller values?
+      if (min) {
+        isExtreme <- (xValues < xRange[idxs,1L]);
+        xRange[idxs[isExtreme],1L] <- xValues[isExtreme];
+      }
 
-      # Identify greater values
-      isExtreme <- (xValues > xRange[idxs,2]);
-      xRange[idxs[isExtreme],2] <- xValues[isExtreme];
+      # Identify greater values?
+      if (max) {
+        isExtreme <- (xValues > xRange[idxs,2L]);
+        xRange[idxs[isExtreme],2L] <- xValues[isExtreme];
+      }
     }
   }
 
+  xRange <- xRange[,which,drop=drop];
+
   # Return data type consistent with range()
   if (storage.mode(x) %in% c("logical", "integer")) {
     if (!any(is.infinite(xRange)))
@@ -166,45 +222,56 @@ setMethod("rowRanges", signature(x="matrix"), function(x, na.rm=FALSE, ...) {
   }
 
   xRange;
-}) # rowRanges()
+} # .rowRanges()
 
+.colRanges <- function(x, na.rm=FALSE, which=1:2, drop=TRUE, ...) {
+  na.rm <- as.logical(na.rm);
+  dim <- dim(x);
+  nrow <- dim[1L];
+  ncol <- dim[2L];
 
-setGeneric("colRanges", function(x, na.rm=FALSE, ...) {
-  standardGeneric("colRanges")
-})
+  # Default range is (+Inf,-Inf). See explanation in help(min)
 
-setMethod("colRanges", signature(x="matrix"), function(x, na.rm=FALSE, ...) {
-  # Use the much faster rowOrderStats() if possible
-  if (!anyMissing(as.vector(x))) {
-    x <- t(x);
-    xRange <- rowOrderStats(x, which=1);
-    xRange <- c(xRange, rowOrderStats(x, which=ncol(x)));
-    dim(xRange) <- c(nrow(x), 2L);
+  # Special cases: 0xN and 1xN matrices
+  if (nrow == 0L) {
+    xRange <- matrix(c(Inf,-Inf), nrow=ncol, ncol=2L, byrow=TRUE);
+    xRange <- xRange[,which,drop=drop];
+    return(xRange);
+  } else if (nrow == 1L) {
+    xRange <- matrix(x[1,,drop=TRUE], nrow=ncol, ncol=2L);
+    if (na.rm) {
+      todo <- which(is.na(xRange[,1L]));
+      ntodo <- length(todo);
+      if (ntodo > 0L) {
+        xRange[todo,] <- matrix(c(Inf,-Inf),
+                                nrow=ntodo, ncol=2L, byrow=TRUE);
+      }
+    }
+    xRange <- xRange[,which,drop=drop];
     return(xRange);
   }
 
-  na.rm <- as.logical(na.rm);
-  ncol <- ncol(x);
-  nrow <- nrow(x);
+  min <- (is.element(1L, which));
+  max <- (is.element(2L, which));
 
-  if (nrow == 0) {
-    # Default range is (+Inf,-Inf).  See explanation in help(min)
-    xRange <- matrix(c(Inf,-Inf), nrow=ncol, ncol=2, byrow=TRUE);
-    return(xRange);
-  } else if (nrow == 1) {
-    xRange <- matrix(x[1,,drop=TRUE], nrow=ncol, ncol=2);
-    if (na.rm) {
-      todo <- which(is.na(xRange[,1]));
-      if (length(todo) > 0) {
-        xRange[todo,] <- matrix(c(Inf,-Inf),
-                             nrow=length(todo), ncol=2, byrow=TRUE);
-      }
+  # Use the much faster rowOrderStats() if possible
+  if (!anyMissing(x)) {
+    x <- t(x);
+    xRange <- NULL;
+    if (min) {
+      xRange <- c(xRange, rowOrderStats(x, which=1L));
+    }
+    if (max) {
+      xRange <- c(xRange, rowOrderStats(x, which=ncol(x)));
+    }
+    if (!drop || length(which) > 1L) {
+      dim(xRange) <- c(nrow(x), length(which));
     }
     return(xRange);
   }
 
   # Default range is (+Inf,-Inf).  See explanation in help(min)
-  xRange <- matrix(c(Inf,-Inf), nrow=ncol, ncol=2, byrow=TRUE);
+  xRange <- matrix(c(Inf,-Inf), nrow=ncol, ncol=2L, byrow=TRUE);
 
   if (na.rm) {
     anyMissing <- TRUE;  # Either none or both of xMin & xMax are NAs.
@@ -213,8 +280,8 @@ setMethod("colRanges", signature(x="matrix"), function(x, na.rm=FALSE, ...) {
 
       # Always update the elements with missing ranges.
       if (anyMissing) {
-        todo <- is.na(xRange[,1]);  # Don't need column 2.
-        anyMissing <- (sum(todo) > 0);
+        todo <- is.na(xRange[,1L]);  # Don't need column 2.
+        anyMissing <- (sum(todo) > 0L);
         if (anyMissing) {
           xRange[todo,1:2] <- xValues[todo];
         }
@@ -229,21 +296,25 @@ setMethod("colRanges", signature(x="matrix"), function(x, na.rm=FALSE, ...) {
       idxs <- which(todo);
       xValues <- xValues[idxs];
 
-      # Identify smaller values
-      isExtreme <- (xValues < xRange[idxs,1]);
-      xRange[idxs[isExtreme],1] <- xValues[isExtreme];
+      # Identify smaller values?
+      if (min) {
+        isExtreme <- (xValues < xRange[idxs,1L]);
+        xRange[idxs[isExtreme],1L] <- xValues[isExtreme];
+      }
 
-      # Identify greater values
-      isExtreme <- (xValues > xRange[idxs,2]);
-      xRange[idxs[isExtreme],2] <- xValues[isExtreme];
+      # Identify greater values?
+      if (max) {
+        isExtreme <- (xValues > xRange[idxs,2L]);
+        xRange[idxs[isExtreme],2L] <- xValues[isExtreme];
+      }
     }
   } else {
     for (rr in 1:nrow) {
       # Update only elements with non-missing extremes
-      idxs <- which(!is.na(xRange[,1]));
+      idxs <- which(!is.na(xRange[,1L]));
 
       # Done?
-      if (length(idxs) == 0)
+      if (length(idxs) == 0L)
         break;
 
       # Assign NAs?
@@ -255,21 +326,27 @@ setMethod("colRanges", signature(x="matrix"), function(x, na.rm=FALSE, ...) {
       idxs <- idxs[!isMissing];
 
       # Done?
-      if (length(idxs) == 0)
+      if (length(idxs) == 0L)
         break;
 
       xValues <- x[rr,idxs];
 
-      # Identify smaller values
-      isExtreme <- (xValues < xRange[idxs,1]);
-      xRange[idxs[isExtreme],1] <- xValues[isExtreme];
+      # Identify smaller values?
+      if (min) {
+        isExtreme <- (xValues < xRange[idxs,1L]);
+        xRange[idxs[isExtreme],1L] <- xValues[isExtreme];
+      }
 
-      # Identify greater values
-      isExtreme <- (xValues > xRange[idxs,2]);
-      xRange[idxs[isExtreme],2] <- xValues[isExtreme];
+      # Identify greater values?
+      if (max) {
+        isExtreme <- (xValues > xRange[idxs,2L]);
+        xRange[idxs[isExtreme],2L] <- xValues[isExtreme];
+      }
     }
   }
 
+  xRange <- xRange[,which,drop=drop];
+
   # Return data type consistent with range()
   if (storage.mode(x) %in% c("logical", "integer")) {
     if (!any(is.infinite(xRange)))
@@ -277,30 +354,17 @@ setMethod("colRanges", signature(x="matrix"), function(x, na.rm=FALSE, ...) {
   }
 
   xRange;
-}) # colRanges()
+} # .colRanges()
 
 
 
 
-rowMins <- function(x, na.rm=FALSE, ...) {
-  rowRanges(x, na.rm=na.rm, ...)[,1];
-}
-
-rowMaxs <- function(x, na.rm=FALSE, ...) {
-  rowRanges(x, na.rm=na.rm, ...)[,2];
-}
-
-colMins <- function(x, na.rm=FALSE, ...) {
-  colRanges(x, na.rm=na.rm, ...)[,1];
-}
-
-colMaxs <- function(x, na.rm=FALSE, ...) {
-  colRanges(x, na.rm=na.rm, ...)[,2];
-}
-
-
 ############################################################################
 # HISTORY:
+# 2013-07-28
+# o SPEEDUP: Made (col|row)Mins() and (col|row)Maxs() faster.
+# o BUG FIX: rowRanges(x) on an Nx0 matrix 'x' would give an error.
+#   Ditto for colRanges(x).
 # 2009-02-01
 # o BUG FIX: colRanges(x) would give an error if nrow(x) == 0.
 # 2008-03-25
