@@ -4,21 +4,25 @@ library("stats")
 # - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
 # Naive R implementation of binMeans()
 # - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
-binMeans0 <- function(y, x, bx) {
+binMeans0 <- function(y, x, bx, count=TRUE, right=FALSE) {
   B <- length(bx)-1L
   res <- double(B)
   counts <- integer(B)
 
   # For each bin...
   for (kk in seq(length=B)) {
-    idxs <- which(bx[kk] <= x & x < bx[kk+1L])
+    if (right) {
+      idxs <- which(bx[kk] <  x & x <= bx[kk+1L])
+    } else {
+      idxs <- which(bx[kk] <= x & x <  bx[kk+1L])
+    }
     yKK <- y[idxs]
     muKK <- mean(yKK)
     res[kk] <- muKK
     counts[kk] <- length(idxs)
   } # for (kk ...)
 
-  attr(res, "count") <- counts
+  if (count) attr(res, "count") <- counts
   res
 } # binMeans0()
 
@@ -39,13 +43,16 @@ bx <- c(0.5,50.5,100.5,150.5,200.5)
 
 yS0 <- binMeans0(y, x=x, bx=bx)
 yS <- binMeans(y, x=x, bx=bx)
-ySr <- rev(binMeans(y, x=-x, bx=rev(-bx), right=TRUE))
 nS <- binCounts(x, bx=bx)
-
 # Sanity check
 stopifnot(all.equal(yS, yS0))
 stopifnot(all(attr(yS, "count"), nS))
-stopifnot(all.equal(ySr, yS, check.attributes=FALSE))
+
+yS0r <- rev(binMeans0(y, x=-x, bx=rev(-bx), count=FALSE, right=TRUE))
+ySr <- rev(binMeans(y, x=-x, bx=rev(-bx), count=FALSE, right=TRUE))
+# Sanity check
+stopifnot(all.equal(yS0r, yS0, check.attributes=FALSE))
+stopifnot(all.equal(ySr, yS0r))
 
 
 # - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
@@ -59,15 +66,9 @@ nb <- 50
 bx <- do.call(seq, c(as.list(range(x)), length.out=nb))
 bx1 <- c(bx[-1], bx[nb] + 1)
 
-t0 <- system.time({
-  yS0 <- binMeans0(y, x=x, bx=bx1)
-})
-
-t <- system.time({
-  yS <- binMeans(y, x=x, bx=bx1)
-})
+yS0 <- binMeans0(y, x=x, bx=bx1)
+yS <- binMeans(y, x=x, bx=bx1)
 nS <- binCounts(x, bx=bx1)
-
 ySr <- rev(binMeans(y, x=-x, bx=rev(-bx1), right=TRUE))
 
 # Sanity check
